@@ -11,9 +11,10 @@
 // - the copyright holder be liable for any  direct, indirect, incidental or -
 // - special damages arising in any way out of the use of this software.     -
 // ---------------------------------------------------------------------------
-// - copyright (c) 1999-2001 amaury darsch                                   -
+// - copyright (c) 1999-2003 amaury darsch                                   -
 // ---------------------------------------------------------------------------
 
+#include "Enum.hpp"
 #include "Real.hpp"
 #include "Meta.hpp"
 #include "List.hpp"
@@ -74,116 +75,120 @@ namespace aleph {
     Nameset* nset = interp->getgset ();
 
     // bind standard constants
-    nset->symcst  ("...",         nset);
-    nset->symcst  ("nil",         (Object*) nilp);
-    nset->symcst  ("true",        new Boolean (true));
-    nset->symcst  ("false",       new Boolean (false));
+    nset->symcst  ("...",           nset);
+    nset->symcst  ("nil",           (Object*) nilp);
+    nset->symcst  ("true",          new Boolean (true));
+    nset->symcst  ("false",         new Boolean (false));
 
     //  builtin functions
-    interp->mkrsv ("if",          new Function (builtin_if));
-    interp->mkrsv ("do",          new Function (builtin_do));
-    interp->mkrsv ("for",         new Function (builtin_for));
-    interp->mkrsv ("try",         new Function (builtin_try));
-    interp->mkrsv ("eval",        new Function (builtin_eval));
-    interp->mkrsv ("sync",        new Function (builtin_sync));
-    interp->mkrsv ("loop",        new Function (builtin_loop));
-    interp->mkrsv ("const",       new Function (builtin_const));
-    interp->mkrsv ("trans",       new Function (builtin_trans));
-    interp->mkrsv ("class",       new Function (builtin_class));
-    interp->mkrsv ("block",       new Function (builtin_block));
-    interp->mkrsv ("while",       new Function (builtin_while));
-    interp->mkrsv ("gamma",       new Function (builtin_gamma));
-    interp->mkrsv ("throw",       new Function (builtin_throw));
-    interp->mkrsv ("force",       new Function (builtin_force));
-    interp->mkrsv ("delay",       new Function (builtin_delay));
-    interp->mkrsv ("lambda",      new Function (builtin_lambda));
-    interp->mkrsv ("switch",      new Function (builtin_switch));
-    interp->mkrsv ("return",      new Function (builtin_return));
-    interp->mkrsv ("launch",      new Function (builtin_launch));
-    interp->mkrsv ("daemon",      new Function (builtin_daemon));
-    interp->mkrsv ("protect",     new Function (builtin_protect));
-    interp->mkrsv ("nameset",     new Function (builtin_nameset));
+    interp->mkrsv ("if",            new Function (builtin_if));
+    interp->mkrsv ("do",            new Function (builtin_do));
+    interp->mkrsv ("for",           new Function (builtin_for));
+    interp->mkrsv ("try",           new Function (builtin_try));
+    interp->mkrsv ("eval",          new Function (builtin_eval));
+    interp->mkrsv ("sync",          new Function (builtin_sync));
+    interp->mkrsv ("loop",          new Function (builtin_loop));
+    interp->mkrsv ("enum",          new Function (builtin_enum));
+    interp->mkrsv ("const",         new Function (builtin_const));
+    interp->mkrsv ("trans",         new Function (builtin_trans));
+    interp->mkrsv ("class",         new Function (builtin_class));
+    interp->mkrsv ("block",         new Function (builtin_block));
+    interp->mkrsv ("while",         new Function (builtin_while));
+    interp->mkrsv ("gamma",         new Function (builtin_gamma));
+    interp->mkrsv ("throw",         new Function (builtin_throw));
+    interp->mkrsv ("force",         new Function (builtin_force));
+    interp->mkrsv ("delay",         new Function (builtin_delay));
+    interp->mkrsv ("lambda",        new Function (builtin_lambda));
+    interp->mkrsv ("switch",        new Function (builtin_switch));
+    interp->mkrsv ("return",        new Function (builtin_return));
+    interp->mkrsv ("launch",        new Function (builtin_launch));
+    interp->mkrsv ("daemon",        new Function (builtin_daemon));
+    interp->mkrsv ("protect",       new Function (builtin_protect));
+    interp->mkrsv ("nameset",       new Function (builtin_nameset));
 
     // builtin operators
-    interp->mkrsv ("+",           new Function (builtin_add));
-    interp->mkrsv ("-",           new Function (builtin_sub));
-    interp->mkrsv ("*",           new Function (builtin_mul));
-    interp->mkrsv ("/",           new Function (builtin_div));
-    interp->mkrsv ("==",          new Function (builtin_eql));
-    interp->mkrsv ("!=",          new Function (builtin_neq));
-    interp->mkrsv (">=",          new Function (builtin_geq));
-    interp->mkrsv (">",           new Function (builtin_gth));
-    interp->mkrsv ("<=",          new Function (builtin_leq));
-    interp->mkrsv ("<",           new Function (builtin_lth));
-    interp->mkrsv ("assert",      new Function (builtin_assert));
+    interp->mkrsv ("+",             new Function (builtin_add));
+    interp->mkrsv ("-",             new Function (builtin_sub));
+    interp->mkrsv ("*",             new Function (builtin_mul));
+    interp->mkrsv ("/",             new Function (builtin_div));
+    interp->mkrsv ("==",            new Function (builtin_eql));
+    interp->mkrsv ("!=",            new Function (builtin_neq));
+    interp->mkrsv (">=",            new Function (builtin_geq));
+    interp->mkrsv (">",             new Function (builtin_gth));
+    interp->mkrsv ("<=",            new Function (builtin_leq));
+    interp->mkrsv ("<",             new Function (builtin_lth));
+    interp->mkrsv ("assert",        new Function (builtin_assert));
 
     // logical operators
-    interp->mkrsv ("or",          new Function (builtin_or));
-    interp->mkrsv ("not",         new Function (builtin_not));
-    interp->mkrsv ("and",         new Function (builtin_and));
+    interp->mkrsv ("or",            new Function (builtin_or));
+    interp->mkrsv ("not",           new Function (builtin_not));
+    interp->mkrsv ("and",           new Function (builtin_and));
     
     // standard printer objects
-    interp->mkrsv ("print",       new Printer  (Printer::OUTPUT));
-    interp->mkrsv ("println",     new Printer  (Printer::OUTPUTLN));
-    interp->mkrsv ("error",       new Printer  (Printer::ERROR));
-    interp->mkrsv ("errorln",     new Printer  (Printer::ERRORLN));
+    interp->mkrsv ("print",         new Printer  (Printer::OUTPUT));
+    interp->mkrsv ("println",       new Printer  (Printer::OUTPUTLN));
+    interp->mkrsv ("error",         new Printer  (Printer::ERROR));
+    interp->mkrsv ("errorln",       new Printer  (Printer::ERRORLN));
 
     // standard predicates
-    interp->mkrsv ("nil-p",       new Function (builtin_nilp));
-    interp->mkrsv ("cons-p",      new Function (builtin_consp));
-    interp->mkrsv ("list-p",      new Function (builtin_listp));
-    interp->mkrsv ("real-p",      new Function (builtin_realp));
-    interp->mkrsv ("node-p",      new Function (builtin_nodep));
-    interp->mkrsv ("edge-p",      new Function (builtin_edgep));
-    interp->mkrsv ("graph-p",     new Function (builtin_graphp));
-    interp->mkrsv ("regex-p",     new Function (builtin_regexp));
-    interp->mkrsv ("queue-p",     new Function (builtin_queuep));
-    interp->mkrsv ("class-p",     new Function (builtin_clsp));
-    interp->mkrsv ("string-p",    new Function (builtin_strp));
-    interp->mkrsv ("thread-p",    new Function (builtin_thrp));
-    interp->mkrsv ("vector-p",    new Function (builtin_vecp));
-    interp->mkrsv ("bitset-p",    new Function (builtin_bitsp));
-    interp->mkrsv ("number-p",    new Function (builtin_nump));
-    interp->mkrsv ("symbol-p",    new Function (builtin_symp));
-    interp->mkrsv ("lexical-p",   new Function (builtin_lexp));
-    interp->mkrsv ("condvar-p",   new Function (builtin_condp));
-    interp->mkrsv ("promise-p",   new Function (builtin_prmp));
-    interp->mkrsv ("boolean-p",   new Function (builtin_boolp));
-    interp->mkrsv ("integer-p",   new Function (builtin_intp));
-    interp->mkrsv ("relatif-p",   new Function (builtin_rltp));
-    interp->mkrsv ("literal-p",   new Function (builtin_litp));
-    interp->mkrsv ("closure-p",   new Function (builtin_clop));
-    interp->mkrsv ("nameset-p",   new Function (builtin_nstp));
-    interp->mkrsv ("instance-p",  new Function (builtin_instp));
-    interp->mkrsv ("resolver-p",  new Function (builtin_rslvp));
-    interp->mkrsv ("hashtable-p", new Function (builtin_ashp));
-    interp->mkrsv ("character-p", new Function (builtin_charp));
-    interp->mkrsv ("qualified-p", new Function (builtin_qualp));
-    interp->mkrsv ("librarian-p", new Function (builtin_lbrnp));
+    interp->mkrsv ("nil-p",         new Function (builtin_nilp));
+    interp->mkrsv ("cons-p",        new Function (builtin_consp));
+    interp->mkrsv ("list-p",        new Function (builtin_listp));
+    interp->mkrsv ("real-p",        new Function (builtin_realp));
+    interp->mkrsv ("node-p",        new Function (builtin_nodep));
+    interp->mkrsv ("edge-p",        new Function (builtin_edgep));
+    interp->mkrsv ("enum-p",        new Function (builtin_enump));
+    interp->mkrsv ("item-p",        new Function (builtin_itemp));
+    interp->mkrsv ("graph-p",       new Function (builtin_graphp));
+    interp->mkrsv ("regex-p",       new Function (builtin_regexp));
+    interp->mkrsv ("queue-p",       new Function (builtin_queuep));
+    interp->mkrsv ("class-p",       new Function (builtin_clsp));
+    interp->mkrsv ("string-p",      new Function (builtin_strp));
+    interp->mkrsv ("thread-p",      new Function (builtin_thrp));
+    interp->mkrsv ("vector-p",      new Function (builtin_vecp));
+    interp->mkrsv ("bitset-p",      new Function (builtin_bitsp));
+    interp->mkrsv ("number-p",      new Function (builtin_nump));
+    interp->mkrsv ("symbol-p",      new Function (builtin_symp));
+    interp->mkrsv ("lexical-p",     new Function (builtin_lexp));
+    interp->mkrsv ("condvar-p",     new Function (builtin_condp));
+    interp->mkrsv ("promise-p",     new Function (builtin_prmp));
+    interp->mkrsv ("boolean-p",     new Function (builtin_boolp));
+    interp->mkrsv ("integer-p",     new Function (builtin_intp));
+    interp->mkrsv ("relatif-p",     new Function (builtin_rltp));
+    interp->mkrsv ("literal-p",     new Function (builtin_litp));
+    interp->mkrsv ("closure-p",     new Function (builtin_clop));
+    interp->mkrsv ("nameset-p",     new Function (builtin_nstp));
+    interp->mkrsv ("instance-p",    new Function (builtin_instp));
+    interp->mkrsv ("resolver-p",    new Function (builtin_rslvp));
+    interp->mkrsv ("hashtable-p",   new Function (builtin_ashp));
+    interp->mkrsv ("character-p",   new Function (builtin_charp));
+    interp->mkrsv ("qualified-p",   new Function (builtin_qualp));
+    interp->mkrsv ("librarian-p",   new Function (builtin_lbrnp));
 
     // standard objects
-    interp->mkrsv ("Real",        new Meta (Real::mknew));
-    interp->mkrsv ("Cons",        new Meta (Cons::mknew));
-    interp->mkrsv ("List",        new Meta (List::mknew));
-    interp->mkrsv ("Node",        new Meta (Node::mknew));
-    interp->mkrsv ("Edge",        new Meta (Edge::mknew));
-    interp->mkrsv ("Regex",       new Meta (Regex::mknew));
-    interp->mkrsv ("Graph",       new Meta (Graph::mknew));
-    interp->mkrsv ("Queue",       new Meta (Queue::mknew));
-    interp->mkrsv ("String",      new Meta (String::mknew));
-    interp->mkrsv ("Buffer",      new Meta (Buffer::mknew));
-    interp->mkrsv ("Vector",      new Meta (Vector::mknew));
-    interp->mkrsv ("BitSet",      new Meta (BitSet::mknew));
-    interp->mkrsv ("Symbol",      new Meta (Symbol::mknew));
-    interp->mkrsv ("Lexical",     new Meta (Lexical::mknew));
-    interp->mkrsv ("Condvar",     new Meta (Condvar::mknew));    
-    interp->mkrsv ("Relatif",     new Meta (Relatif::mknew));    
-    interp->mkrsv ("Integer",     new Meta (Integer::mknew));
-    interp->mkrsv ("Boolean",     new Meta (Boolean::mknew));
-    interp->mkrsv ("Closure",     new Meta (Closure::mknew));
-    interp->mkrsv ("Resolver",    new Meta (Resolver::mknew));
-    interp->mkrsv ("Character",   new Meta (Character::mknew));
-    interp->mkrsv ("Librarian",   new Meta (Librarian::mknew));
+    interp->mkrsv ("Enum",          new Meta (Enum::mknew));
+    interp->mkrsv ("Real",          new Meta (Real::mknew));
+    interp->mkrsv ("Cons",          new Meta (Cons::mknew));
+    interp->mkrsv ("List",          new Meta (List::mknew));
+    interp->mkrsv ("Node",          new Meta (Node::mknew));
+    interp->mkrsv ("Edge",          new Meta (Edge::mknew));
+    interp->mkrsv ("Regex",         new Meta (Regex::mknew));
+    interp->mkrsv ("Graph",         new Meta (Graph::mknew));
+    interp->mkrsv ("Queue",         new Meta (Queue::mknew));
+    interp->mkrsv ("String",        new Meta (String::mknew));
+    interp->mkrsv ("Buffer",        new Meta (Buffer::mknew));
+    interp->mkrsv ("Vector",        new Meta (Vector::mknew));
+    interp->mkrsv ("BitSet",        new Meta (BitSet::mknew));
+    interp->mkrsv ("Symbol",        new Meta (Symbol::mknew));
+    interp->mkrsv ("Lexical",       new Meta (Lexical::mknew));
+    interp->mkrsv ("Condvar",       new Meta (Condvar::mknew));    
+    interp->mkrsv ("Relatif",       new Meta (Relatif::mknew));    
+    interp->mkrsv ("Integer",       new Meta (Integer::mknew));
+    interp->mkrsv ("Boolean",       new Meta (Boolean::mknew));
+    interp->mkrsv ("Closure",       new Meta (Closure::mknew));
+    interp->mkrsv ("Resolver",      new Meta (Resolver::mknew));
+    interp->mkrsv ("Character",     new Meta (Character::mknew));
+    interp->mkrsv ("Librarian",     new Meta (Librarian::mknew));
   }
 
   // this procedure look for a shared library
@@ -217,6 +222,7 @@ namespace aleph {
     // initialize default values
     d_assert = false;
     d_cloned = false;
+    d_next   = false;
     // create a default terminal
     Object::iref (p_term = new Terminal);
     // save streams
@@ -242,6 +248,8 @@ namespace aleph {
     Object::iref (p_shlib = new Vector);
     // create the vector of namesets
     Object::iref (p_vnset = new Vector);
+    // bind the main thread
+    Thread::setrobj (this);
   }
 
   // create a new interpreter with the specified streams
@@ -250,6 +258,7 @@ namespace aleph {
     // initialize default values
     d_assert = false;
     d_cloned = false;
+    d_next   = false;
     p_term   = nilp;
     // save streams
     Object::iref (p_is = is);
@@ -274,6 +283,8 @@ namespace aleph {
     Object::iref (p_shlib = new Vector);
     // create the vector of namesets
     Object::iref (p_vnset = new Vector);
+    // bind the main thread
+    Thread::setrobj (this);
   }
 
   // copy constructor
@@ -282,6 +293,7 @@ namespace aleph {
     // initialize default value
     d_assert = that.d_assert;
     d_cloned = true;
+    d_next   = false;
     // copy the terminal
     Object::iref (p_term = that.p_term);
     // copy the streams - but not the input one
@@ -487,11 +499,33 @@ namespace aleph {
   // set the interpreter arguments
 
   void Interp::setargs (const Strvec& args) {
+    // reset the vector
+    p_argv->reset ();
+    // fill in the arguments
     long len = args.length ();
     for (long i = 0; i < len; i++) p_argv->append (new String (args.get (i)));
   }
 
-  // add a new path entry for the interpreter
+  // get the interpreter arguments
+
+  Strvec Interp::getargs (void) const {
+    Strvec result;
+    long len = p_argv->length ();
+    for (long i = 0; i < len; i++) {
+      String* str = dynamic_cast <String*> (p_argv->get (i));
+      if (str == nilp) continue;
+      result.add (*str);
+    }
+    return result;
+  }
+
+  // add a new path entry in the resolver
+
+  void Interp::addpath (const String& path) {
+    p_rslv->add (path);
+  }
+
+  // set the path resolver
 
   void Interp::setpath (const Strvec& path) {
     long len = path.length ();
@@ -549,39 +583,80 @@ namespace aleph {
       throw;
     }
   }
- 
-  // loop on the standard input by doing a read-eval loop
-  // this procedure return false is something bad happen
 
-  bool Interp::loop (void) {
+  // set the next flag
+  
+  void Interp::setnext (const bool flag) {
+    wrlock ();
+    d_next = flag;
+    unlock ();
+  }
+
+  // return the next flag
+
+  bool Interp::getnext (void) const {
+    rdlock ();
+    bool result = d_next;
+    unlock ();
+    return result;
+  }
+
+  // break the interpreter in a nameset with an object
+
+  bool Interp::bpt (Nameset* nset, Object* object) {
+    if (object != nilp) object->eval (this, nset);
+    return true;
+  }
+
+  // loop on the standard input in a nameset context
+
+  bool Interp::loop (Nameset* nset) {
+    // initialize the status
     bool status = true;
     // create a new reader 
     Reader* rd = new Reader (getis ());
     // loop until we have an eof
     while (true) {
-      Cons* cons = nilp;
+      Form* form = nilp;
       try {
-	cons = rd->parse ();
-	if (cons == nilp) break;
-	Object::cref (cons->eval (this, p_gset));
-	Object::dref (cons);
+	form = rd->parse ();
+	if (form == nilp) break;
+	Object::cref (form->eval (this, p_gset));
+	Object::dref (form);
       } catch (const Exception& e) {
-	Object::dref (cons);
+	if (e.getabf () == true) throw;
 	status = false;
 	getes()->errorln (e);
-	if (e.getabf () == true) System::exit (1);
+	Object::dref (form);
       } catch (const Return& r) {
-	Object::dref (cons);
 	this->post (r.getobj ());
+	Object::dref (form);
       } catch (...) {
-	Object::dref (cons);
-	status = false;
-	getes()->errorln ("fatal: unknown exception trapped");
-	break;
+	Object::dref (form);
+	delete rd;
+	throw;
       }
     }
-    // clean and return
     delete rd;
+    return status;
+  }
+
+  // loop on the standard input by doing a read-eval loop
+  // this procedure return false is something bad happen
+
+  bool Interp::loop (void) {
+    // initialize status
+    bool status = true;
+    // loop in the standard input
+    try {
+      status = loop (p_gset);
+    } catch (const Exception& e) {
+      getes()->errorln (e);
+      if (e.getabf () == true) System::exit (1);
+    } catch (...) {
+      status = false;
+      getes()->errorln ("fatal: unknown exception trapped");
+    }
     // wait for all threads to terminate - this works only for the
     // master thread - for other threads, this does nothing
     Thread::waitall ();
@@ -592,30 +667,35 @@ namespace aleph {
   
   void Interp::load (const String& fname) {
     // try to open the module
-    Module* mp = new Module (p_rslv->alplkp (fname));
+    Input*  ms = p_rslv->alplkp  (fname);
+    String  mn = p_rslv->alpname (fname);
+    Module* mp = new Module (ms, mn);
     // loop until we have an eof
     while (true) {
-      Cons* cons = nilp;
+      Form* form = nilp;
       try {
-	cons = mp->parse ();
-	if (cons == nilp) break;
-	Object::cref (cons->eval (this, p_gset));
-	Object::dref (cons);
+	form = mp->parse ();
+	if (form == nilp) break;
+	Object::cref (form->eval (this, p_gset));
+	Object::dref (form);
       } catch (Exception& e) {
 	e.setfname (fname);
-	e.setlnum  (mp->getlnum ());
+	if (form == nilp)
+	  e.setlnum (mp->getlnum ());
+	else
+	  e.setlnum (form->getlnum ());
 	delete mp;
 	throw;
       } catch (const Return& r) {
-	Object::dref (cons);
+	Object::dref (form);
 	this->post (r.getobj ());
       } catch (...) {
-	Object::dref (cons);
+	Object::dref (form);
 	delete mp;
 	throw;
       }
     }
-    // clean the module and return status
+    // clean the module
     delete mp;
   }
 
@@ -644,7 +724,9 @@ namespace aleph {
   
   void Interp::compile (const String& name, Output& os) {
     // try to open the module and write
-    Module* mp = new Module (p_rslv->lookup (name));
+    Input*  ms = p_rslv->lookup  (name);
+    String  mn = p_rslv->getpath (name);
+    Module* mp = new Module (ms, mn);
     try {
       mp->write (os);
     } catch (const Exception& e) {
