@@ -11,7 +11,7 @@
 // - the copyright holder be liable for any  direct, indirect, incidental or -
 // - special damages arising in any way out of the use of this software.     -
 // ---------------------------------------------------------------------------
-// - copyright (c) 1999-2001 amaury darsch                                   -
+// - copyright (c) 1999-2003 amaury darsch                                   -
 // ---------------------------------------------------------------------------
 
 #include "cnet.hpp"
@@ -151,6 +151,9 @@ namespace aleph {
     lval.l_linger = ival;
     // dispatch ip independant options
     switch (opt) {
+    case SOCKET_REUSEADDR:
+      return c_setsockopt (sid, SOL_SOCKET, SO_REUSEADDR,&bval,sizeof (bval));
+      break;
     case SOCKET_DONTROUTE:
       return c_setsockopt (sid, SOL_SOCKET, SO_DONTROUTE,&bval,sizeof (bval));
       break;
@@ -284,6 +287,9 @@ namespace aleph {
     lval.l_linger = ival;
     // dispatch ip independant options
     switch (opt) {
+    case SOCKET_REUSEADDR:
+      return c_setsockopt (sid, SOL_SOCKET, SO_REUSEADDR,&bval,sizeof (bval));
+      break;
     case SOCKET_BROADCAST:
       return c_setsockopt (sid, SOL_SOCKET, SO_BROADCAST,&bval,sizeof (bval));
       break;
@@ -382,14 +388,24 @@ namespace aleph {
 #endif
 
 namespace aleph {
-  // mutex for dns-service query
-  static void* mtx = nilp;
+  // static mutex creation function
+  static void* mtx_create (void);
+  // mutex or network services
+  static void* mtx = mtx_create ();
 
   // this function destroy the mutex at exit
   static void mtx_destroy (void) {
     c_mtxdestroy (mtx);
   }
-  
+
+  // this function initialize a mutex statically and register its
+  // destruction to be done at exit
+  static void* mtx_create (void) {
+    void* mtx = c_mtxcreate ();
+    c_atexit (mtx_destroy);
+    return mtx;
+  }
+
   // return the loopback name
 
   const char* c_loopname (void) {
@@ -434,11 +450,7 @@ namespace aleph {
   char* c_ipcanon (const char* name) {
     // check for valid host name
     if (c_strlen (name) == 0) return nilp;
-    // get the mutex if needed
-    if (mtx == nilp) {
-      mtx = c_mtxcreate ();
-      c_atexit (mtx_destroy);
-    }
+    // lock the mutex
     c_mtxlock (mtx);
     // call the resolver and get address
     struct hostent* hostrec = gethostbyname (name);
@@ -457,11 +469,7 @@ namespace aleph {
   t_byte* c_ipaddr (const char* name) {
     // standard check as usual
     if (c_strlen (name) == 0) return nilp;
-    // get the mutex if needed
-    if (mtx == nilp) {
-      mtx = c_mtxcreate ();
-      c_atexit (mtx_destroy);
-    }
+    // lock the mutex
     c_mtxlock (mtx);  
     // call the resolver and query everythinng we need
     struct hostent* hostrec = gethostbyname (name);
@@ -482,16 +490,24 @@ namespace aleph {
     return result;
   }
 
+  // make a copy of an address
+
+  t_byte* c_cpaddr (const t_byte* addr) {
+    // check for nil address
+    if (addr == nilp) return nilp;
+    // check for length
+    long len = (long) addr[0] + 1;
+     t_byte* result = new t_byte[len];
+     for (long i = 0; i < len; i++) result[i] = addr[i];
+     return result;
+  }
+
   // get a service by name and flag
 
   t_word c_ipserv (const char* name, bool flag) {
     // check for service
     if (name == nilp) return 0;
-    // get the mutex if needed
-    if (mtx == nilp) {
-      mtx = c_mtxcreate ();
-      c_atexit (mtx_destroy);
-    }
+    // lock the mutex
     c_mtxlock (mtx);
     // get the service
     const char* proto  = flag ? "tcp" : "udp";
@@ -582,11 +598,6 @@ namespace aleph {
     mzero (&address, addrlen);
     // check for a valid socket
     if (sid < 0) return false;
-    // set the socket option to reuse the address
-    int  sopt = 1;
-    bool flag = c_setsockopt (sid, SOL_SOCKET, SO_REUSEADDR, &sopt, 
-			      sizeof (sopt));
-    if (flag == false) return false;
     // initialize the address
     init_addr (address, port, c_isipv6 (sid));
     // bind this socket
@@ -602,11 +613,6 @@ namespace aleph {
     mzero (&address, addrlen);
     // check for a valid socket
     if (sid < 0) return false;
-    // set the socket option to reuse the address
-    int  sopt = 1;
-    bool flag = c_setsockopt (sid, SOL_SOCKET, SO_REUSEADDR, &sopt, 
-			      sizeof (sopt));
-    if (flag == false) return false;
     // initialize the address
     byte_to_addr (address, port, dest);
     // bind this socket
