@@ -11,18 +11,17 @@
 // - the copyright holder be liable for any  direct, indirect, incidental or -
 // - special damages arising in any way out of the use of this software.     -
 // ---------------------------------------------------------------------------
-// - copyright (c) 1999-2001 amaury darsch                                   -
+// - copyright (c) 1999-2003 amaury darsch                                   -
 // ---------------------------------------------------------------------------
 
 #include "Cons.hpp"
-#include "Real.hpp"
 #include "Input.hpp"
 #include "Vector.hpp"
 #include "Output.hpp"
 #include "Recycle.hpp"
 #include "Boolean.hpp"
+#include "Integer.hpp"
 #include "Runnable.hpp"
-#include "Character.hpp"
 #include "Exception.hpp"
 
 namespace aleph {
@@ -67,6 +66,7 @@ namespace aleph {
     p_car  = nilp;
     p_cdr  = nilp;
     p_mon  = nilp;
+    d_bpt  = false;
   }
 
   // create a new cons cell with a type
@@ -76,6 +76,7 @@ namespace aleph {
     p_car  = nilp;
     p_cdr  = nilp;
     p_mon  = nilp;
+    d_bpt  = false;
   }
 
   // create a new cons cell with a car
@@ -85,6 +86,7 @@ namespace aleph {
     p_car  = Object::iref (car);
     p_cdr  = nilp;
     p_mon  = nilp;
+    d_bpt  = false;
   }
 
   // create a new cons cell with a type and a car
@@ -94,6 +96,7 @@ namespace aleph {
     p_car  = Object::iref (car);
     p_cdr  = nilp;
     p_mon  = nilp;
+    d_bpt  = false;
   }
 
   // copy constructor for this cons cell
@@ -102,6 +105,7 @@ namespace aleph {
     d_type  = that.d_type;
     p_car   = Object::iref (that.p_car);
     p_cdr   = that.p_cdr;
+    d_bpt   = that.d_bpt;
     Object::iref (that.p_cdr);
     p_mon   = (that.p_mon == nilp) ? nilp : new Monitor;
   }
@@ -187,6 +191,7 @@ namespace aleph {
       d_type = BLOCK;
       break;
     default:
+      unlock ();
       throw Exception ("serial-error", "invalid cons cell type found");
       break;
     }
@@ -234,7 +239,7 @@ namespace aleph {
       // find the last cons cell
       Cons* last = this;
       while (last->p_cdr != nilp) last = last->p_cdr;
-      // create attach this new cons cell
+      // attach this new cons cell
       last->p_cdr = cons;
       Object::iref (cons);
       // unlock the cons cell
@@ -436,7 +441,15 @@ namespace aleph {
   
   void Cons::mksync (void) {
     wrlock ();
-    p_mon = new Monitor;
+    if (p_mon == nilp) p_mon = new Monitor;
+    unlock ();
+  }
+
+  // set the form breakpoint
+
+  void Cons::setbpt (const bool bpt) {
+    wrlock ();
+    d_bpt = bpt;
     unlock ();
   }
 
@@ -500,15 +513,21 @@ namespace aleph {
   // evaluate this cons cell in the current nameset
 
   Object* Cons::eval (Runnable* robj, Nameset* nset) {
+    // check for breakpoint
+    if (d_bpt == true) robj->bpt (nset, this);
     // synchronize the form
     if (p_mon != nilp) p_mon->enter ();
     Object* result = nilp;
     try {
       if (d_type == Cons::BLOCK) {
-	Cons*   cons   = this;
+	Cons* cons = this;
 	while (cons != nilp) {
 	  Object::cref (result);
 	  Object* car = cons->getcar ();
+	  if (robj->getnext () == true) {
+	    robj->setnext (false);
+	    robj->bpt (nset, car);
+	  }
 	  result = (car == nilp) ? nilp : car->eval (robj,nset);
 	  cons   = cons->getcdr ();
 	}
