@@ -11,7 +11,7 @@
 // - the copyright holder be liable for any  direct, indirect, incidental or -
 // - special damages arising in any way out of the use of this software.     -
 // ---------------------------------------------------------------------------
-// - copyright (c) 1999-2001 amaury darsch                                   -
+// - copyright (c) 1999-2003 amaury darsch                                   -
 // ---------------------------------------------------------------------------
 
 #include "Input.hpp"
@@ -56,6 +56,10 @@ namespace aleph {
   static const long QUARK_ADD       = String::intern ("+");
   static const long QUARK_EQL       = String::intern ("==");
   static const long QUARK_NEQ       = String::intern ("!=");
+  static const long QUARK_LTH       = String::intern ("<");
+  static const long QUARK_LEQ       = String::intern ("<=");
+  static const long QUARK_GTH       = String::intern (">");
+  static const long QUARK_GEQ       = String::intern (">=");
   static const long QUARK_AEQ       = String::intern ("+=");
   static const long QUARK_GET       = String::intern ("get");
   static const long QUARK_SPLIT     = String::intern ("split");
@@ -96,7 +100,7 @@ namespace aleph {
   }
   
   // compare two strings - again local inline for performance
-  static inline bool cmpstr (const char* s1, const char* s2) {
+  static inline bool eqlstr (const char* s1, const char* s2) {
     // nilp case first
     const char* ss1 = (s1 == nilp) ? "\0" : s1;
     const char* ss2 = (s2 == nilp) ? "\0" : s2;
@@ -111,6 +115,41 @@ namespace aleph {
     }
     return (*ss1 == *ss2) ? true : false;
   }
+
+  // compare two strings - less than operator
+  static bool lthstr (const char* s1, const char* s2) {
+    // nilp case first
+    const char* ss1 = (s1 == nilp) ? "\0" : s1;
+    const char* ss2 = (s2 == nilp) ? "\0" : s2;
+    
+    // check first character for fast compare
+    if (*ss1 >= *ss2) return false;
+    
+    // normal compare now
+    while (*ss1 != nilc) {
+      if (*ss2 == nilc)     return true;
+      if (*ss1++ >= *ss2++) return true;
+    }
+    if ((*ss1 == nilc) && (*ss2 == nilc)) return true;
+    return (*ss1 < *ss2) ? true : false;
+  }
+
+  // compare two strings - less equal operator
+  static bool leqstr (const char* s1, const char* s2) {
+    // nilp case first
+    const char* ss1 = (s1 == nilp) ? "\0" : s1;
+    const char* ss2 = (s2 == nilp) ? "\0" : s2;
+    
+    // check first character for fast compare
+    if (*ss1 > *ss2) return false;
+    
+    // normal compare now
+    while (*ss1 != nilc) {
+      if (*ss2 == nilc)    return true;
+      if (*ss1++ > *ss2++) return true;
+    }
+    return (*ss1 <= *ss2) ? true : false;
+  }
   
   // create an empty string
   
@@ -355,26 +394,78 @@ namespace aleph {
   // compare a string with another one
   
   bool String::operator == (const String& s) const {
-    return cmpstr (p_string->p_buffer,s.p_string->p_buffer);
+    return eqlstr (p_string->p_buffer,s.p_string->p_buffer);
   }
 
   // compare a string with a c-string
   
   bool String::operator == (const char* s) const {
-    return cmpstr (p_string->p_buffer,s);
+    return eqlstr (p_string->p_buffer,s);
   }
   
   // compare a string with another one
   
   bool String::operator != (const String& s) const {
-    bool result = cmpstr (p_string->p_buffer,s.p_string->p_buffer);
+    bool result = eqlstr (p_string->p_buffer,s.p_string->p_buffer);
     return result ? false : true;
   }
   
   // compare a string with a c-string
   
   bool String::operator != (const char* s) const {
-    bool result = cmpstr (p_string->p_buffer,s);
+    bool result = eqlstr (p_string->p_buffer,s);
+    return result ? false : true;
+  }
+
+  // compare a string with another one
+  
+  bool String::operator < (const String& s) const {
+    return lthstr (p_string->p_buffer,s.p_string->p_buffer);
+  }
+
+  // compare a string with a c-string
+  
+  bool String::operator < (const char* s) const {
+    return lthstr (p_string->p_buffer,s);
+  }
+
+  // compare a string with another one
+  
+  bool String::operator <= (const String& s) const {
+    return leqstr (p_string->p_buffer,s.p_string->p_buffer);
+  }
+
+  // compare a string with a c-string
+  
+  bool String::operator <= (const char* s) const {
+    return leqstr (p_string->p_buffer,s);
+  }
+
+  // compare a string with another one
+  
+  bool String::operator > (const String& s) const {
+    bool result = leqstr (p_string->p_buffer,s.p_string->p_buffer);
+    return result ? false : true;
+  }
+
+  // compare a string with a c-string
+  
+  bool String::operator > (const char* s) const {
+    bool result = leqstr (p_string->p_buffer,s);
+    return result ? false : true;
+  }
+
+  // compare a string with another one
+  
+  bool String::operator >= (const String& s) const {
+    bool result = lthstr (p_string->p_buffer,s.p_string->p_buffer);
+    return result ? false : true;
+  }
+
+  // compare a string with a c-string
+  
+  bool String::operator >= (const char* s) const {
+    bool result = lthstr (p_string->p_buffer,s);
     return result ? false : true;
   }
 
@@ -614,6 +705,18 @@ namespace aleph {
     case Object::NEQ:
       if (sobj != nilp) return new Boolean (*this != *sobj);
       break;
+    case Object::LTH:
+      if (sobj != nilp) return new Boolean (*this < *sobj);
+      break;
+    case Object::LEQ:
+      if (sobj != nilp) return new Boolean (*this <= *sobj);
+      break;
+    case Object::GTH:
+      if (sobj != nilp) return new Boolean (*this > *sobj);
+      break;
+    case Object::GEQ:
+      if (sobj != nilp) return new Boolean (*this >= *sobj);
+      break;
     default:
       throw Exception ("operator-error", "unsupported string operator");
     }
@@ -707,7 +810,7 @@ namespace aleph {
       }
     }
 
-    // call the object method
-    return Object::apply (robj, nset, quark, argv);
+    // call the literal method
+    return Literal::apply (robj, nset, quark, argv);
   }
 }
